package com.ozs.web.core.util;


import com.alibaba.fastjson2.JSON;
import com.ozs.common.config.BaseConfig;
import com.ozs.common.constant.Constants;
import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.exception.base.BaseException;
import com.ozs.common.utils.DateUtils;
import com.ozs.common.utils.http.HttpUtils;
import com.ozs.system.mapper.SysDictDataMapper;
import com.ozs.web.core.config.CaneraConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;

import javax.annotation.PostConstruct;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 相机工具
 */
@Component
@Slf4j
public class CameraUtil {

    private static String historyUrl;
    private static String ffmpegPath;
    private static String filePath;
    private static String transcribeFilePath;
    private static String webUrl;
    private static String bakUrl;

    @Autowired
    private CaneraConfig caneraConfig;
    @Autowired
    private SysDictDataMapper dictDataMapper;


    /**
     * 历史回放
     *
     * @param fromVideoFileList
     * @param ph
     * @return
     * @throws IOException
     */
    public static String historyPlay(List<String> fromVideoFileList, String ph) {
        // 视频服务映射路径
        String NewfilePath = BaseConfig.getProfile() + "/" + ph;
        log.info("NewfilePath:{}", NewfilePath);
        log.info("fromVideoFileList:{}", fromVideoFileList);
        if (ObjectUtils.isEmpty(fromVideoFileList) || fromVideoFileList.size() <= 0) {
            throw new BaseException("当前相机无视频录像");
        }
        new Thread(() -> {
            try {
                myConvetor(fromVideoFileList, NewfilePath);
            } catch (IOException e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }).start();
        return Constants.RESOURCE_PREFIX + "/" + ph;
    }

    /**
     * 实时播放的拼接流
     *
     * @param cameraCode 相机编码
     * @param channel    相机通道
     * @return
     */
    public static String getPlayFlv(String cameraCode, String channel, boolean flay) {
        if (!flay) {
            return bakUrl + "/hdl/" + channel + "/" + cameraCode + ".flv";
        }
        return webUrl + "/hdl/" + channel + "/" + cameraCode + ".flv";
    }

    public static String getPlayFlv(String cameraCode, String channel) {
        return getPlayFlv(cameraCode, channel, true);
    }

    /**
     * 开启录制功能
     *
     * @param cameraCode 相机编码
     * @param channel    相机通道
     * @return
     */
    public static String startRecording(String cameraCode, String channel) {
        return historyUrl + "/api/record/flv/start?streamPath=" + channel + "/" + cameraCode;
    }

    /**
     * 关闭录制功能
     *
     * @param taskId 录像接口返回的任务ID
     * @return
     */
    public static String endRecording(String taskId) {
        return historyUrl + "/api/record/flv/stop?id=" + taskId;
    }


    /**
     * 历史回放流（获取）
     *
     * @param channel 相机通道
     * @param startTm 开始时间
     * @param endTm   结束时间
     * @return
     */
    public static String historyPlayListStr(String channel, Date startTm, Date endTm) {
        List<String> list = filterPlayList(channel, startTm, endTm, filePath);
        String ph = "flv/" + DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, new Date())
                + "/" + channel + "/"
                + UUID.randomUUID().toString() + ".flv";
        if (!ObjectUtils.isEmpty(list)) {
            try {
                return historyPlay(list, ph);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        } else {
            throw new BaseException("当前相机无视频录像");
        }
        return null;
    }


    /**
     * 过滤符合条件的视频
     *
     * @param channel
     * @param startTm
     * @param endTm
     * @param mappingUrl
     * @return
     */
    public static List<String> filterPlayList(String channel, Date startTm, Date endTm, String mappingUrl) {
        if (StringUtils.isBlank(channel)
                || ObjectUtils.isEmpty(startTm)
                || ObjectUtils.isEmpty(endTm)) {
            return null;
        }

        List<String> ls = new ArrayList<>();
        Map<Date, String> m = new HashMap<>();
        // 调用视频服务返回参数
        String startTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, startTm);
        String endTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, endTm);

        String param = "channel=" + channel + "&startTime=" + startTime + "&endTime=" + endTime;
        String s = HttpUtils.sendGet(historyUrl + "/api/record/flv/list", param);
        // 视频拼接
        if (!StringUtils.isBlank(s) || "null".equals(s)) {
            List<Map<String, Object>> maps = JSON.parseArray(s, Map.class);
            if (ObjectUtils.isEmpty(maps)) {
                return null;
            }
            for (Map<String, Object> map : maps) {
                Object path = map.get("Path");
                if (!ObjectUtils.isEmpty(path)) {
                    String s1 = path.toString();
                    String substring = s1.substring(s1.lastIndexOf("/") + 1, s1.length());
                    String substring1 = substring.substring(0, substring.indexOf("-"));
                    String[] s2 = substring1.split("_");
                    if (!ObjectUtils.isEmpty(s2)) {
                        String s3 = s2[0] + s2[1];
                        String s4 = s2[0] + s2[2];
                        Date sdate = DateUtils.dateTime(DateUtils.YYYYMMDDHHMMSS, s3);
                        Date edate = DateUtils.dateTime(DateUtils.YYYYMMDDHHMMSS, s4);
                        /*     sdate  |startTm|  edate     |endTm| */
                        if (startTm.compareTo(sdate) >= 0
                                && startTm.compareTo(edate) <= 0
                                && endTm.compareTo(edate) >= 0) {
                            m.put(sdate, mappingUrl + path.toString());
                            /*  |startTm|   sdate    edate   |endTm| */
                        } else if (startTm.compareTo(sdate) <= 0 && endTm.compareTo(edate) >= 0) {
                            m.put(sdate, mappingUrl + path.toString());

                            /*  |startTm|   sdate     |endTm|  edate  */
                        } else if (startTm.compareTo(sdate) <= 0
                                && endTm.compareTo(sdate) >= 0
                                && endTm.compareTo(edate) <= 0) {
                            m.put(sdate, mappingUrl + path.toString());

                            /*  sdate |startTm|      |endTm| edate  */
                        } else if (startTm.compareTo(sdate) >= 0 && endTm.compareTo(edate) <= 0) {
                            m.put(sdate, mappingUrl + path.toString());
                        }
                    }
                }
            }
        }
        if (!ObjectUtils.isEmpty(m) && m.size() > 0) {
            Set<Date> dates = m.keySet();
            // 排序
            dates.stream().parallel().collect(Collectors.toList()).stream().sorted().forEach(d -> {
                ls.add(m.get(d));
            });
            return ls;
        }
        return null;
    }

    /**
     * 合并视频（转化文件）
     *
     * @param fromVideoFileList 视频路径
     * @param newfilePath       生产新的视频文件路径
     * @throws IOException
     */
    public static void myConvetor(List<String> fromVideoFileList,
                                  String newfilePath) throws IOException {
        /*
        * for f in *.flv; do echo "file '$f'" >> mylist.txt; done
            ffmpeg -f concat -i mylist.txt -c copy output.flv
        * */

        List<String> rspList = new ArrayList<String>();
        File file = new File(newfilePath);
        boolean flay = false;
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
            log.info("创建文件夹：{}", file.getParentFile().getPath());
            flay = true;
        }
        log.info("newfilePath：{}", newfilePath);


        StringBuffer sm = new StringBuffer("for f in ");
        for (int t = 0; t < fromVideoFileList.size(); t++) {
            File ft = new File(fromVideoFileList.get(t));
            if (ft.exists()) {
                if (t != fromVideoFileList.size() - 1) {
                    sm.append(fromVideoFileList.get(t) + " ");
                } else {
                    sm.append(fromVideoFileList.get(t));
                }
            }
        }
        String substring = newfilePath.substring(0, newfilePath.lastIndexOf("."));
        substring = substring + ".txt";
        log.info("临时转化的文件：{}", substring);


        sm.append("; do echo \"file '$f'\" >> " + substring + "; done");
        String commit = sm.toString();
        log.info("合并转化文件的命令：{}", commit);
        Process proc = Runtime.getRuntime().exec("/bin/bash", null, null);
        BufferedReader in = new BufferedReader(new InputStreamReader(proc.getInputStream()));
        PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(proc.getOutputStream())), true);

        try {
            String commit1 = ffmpegPath + " -f concat -safe 0 -i " + substring + " -c copy " + newfilePath;
            List<String> commands = new ArrayList<>();
            if (flay) {
                commands.add("chomd -R 777 " + file.getParentFile().getPath());
            }
            // 合并成一个临时文件
            commands.add(commit);
            // 临时文件转化为 flv
            commands.add(commit1);
            for (String line : commands) {
                out.println(line);
            }
            out.println("exit");// 这个命令必须执行，否则in流不结束。
            String rspLine = "";
            while ((rspLine = in.readLine()) != null) {
                System.out.println(rspLine);
                rspList.add(rspLine);
            }
            int i = proc.waitFor();
            log.info("执行结果：{}", i);
            //删除生成的ts文件
            File file1 = new File(substring);
            if (file1.exists()) {
                file1.delete();
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            in.close();
            out.close();
            proc.destroy();
        }
        log.info("返回结果：{}", rspList);
    }

    public static void main(String[] args) throws IOException {

//        List<String> fromVideoFileList = new ArrayList();
//        String p = "C:\\Users\\Administrator.DESKTOP-0NUUTMM\\Desktop\\work\\106\\流媒体\\";
//        String newfilePath = p + "hb.flv";
//        fromVideoFileList.add(p + "20230303_174931_175031-d6d56396-b9a8-11ed-aeb9-00163e06a5f3.flv");
//        fromVideoFileList.add(p + "20230303_175032_175132-fb32b9fe-b9a8-11ed-aeb9-00163e06a5f3.flv");
//        fromVideoFileList.add(p + "20230303_175133_175233-1f97af04-b9a9-11ed-aeb9-00163e06a5f3.flv");
//        fromVideoFileList.add(p + "20230303_175234_175334-43f4daf1-b9a9-11ed-aeb9-00163e06a5f3.flv");
//        fromVideoFileList.add(p + "m.flv");
//        fromVideoFileList.add(p + "m33.flv");
//        fromVideoFileList.add(p + "mv.flv");
//        fromVideoFileList.add(p + "mvido.flv");
//        convetor(fromVideoFileList, newfilePath);

        String channel = "34020000001320000002";
        String startTime = "2023-03-09";
        String endTime = "2023-03-09";
        String param = "channel=" + channel + "&startTime=" + startTime + "&endTime=" + endTime;
        String s = HttpUtils.sendGet("http://47.106.159.135/streams" + "/api/record/flv/list", param);
    }


    @PostConstruct
    public void init() {
        historyUrl = caneraConfig.getHistoryUrl();
        ffmpegPath = caneraConfig.getFfmpegPath();
        filePath = caneraConfig.getFilePath();
        transcribeFilePath = caneraConfig.getTranscribeFilePath();
        webUrl = caneraConfig.getWebUrl();
        bakUrl = caneraConfig.getBakUrl();
    }

    /**
     * 定时任务参数flv文件
     *
     * @throws IOException
     * @throws InterruptedException
     */
    public void deleteFlv() throws IOException, InterruptedException {
        SimpleDateFormat s = new SimpleDateFormat("yyyy-MM-dd");
        Date date = new Date();
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        calendar.add(calendar.DATE, -1); //把日期往后增加一天,整数  往后推,负数往前移动
        date = calendar.getTime(); //这个时间就是日期往后推一天的结果
        String path = BaseConfig.getProfile() + "/flv/" + DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, date);
        File file = new File(path);
        if (file.exists()) {
            List<String> rspList = new ArrayList<String>();
            Process proc = Runtime.getRuntime().exec("/bin/bash", null, null);
            BufferedReader in = new BufferedReader(new InputStreamReader(proc.getInputStream()));
            PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(proc.getOutputStream())), true);
            String commit = " rm -rf  " + path;
            List<String> commands = new ArrayList<>();

            // 删除
            commands.add(commit);
            for (String line : commands) {
                out.println(line);
            }
            out.println("exit");// 这个命令必须执行，否则in流不结束。
            String rspLine = "";
            while ((rspLine = in.readLine()) != null) {
                System.out.println(rspLine);
                rspList.add(rspLine);
            }
            int i = proc.waitFor();
            log.info("执行结果：{}", i);

            in.close();
            out.close();
            proc.destroy();
        }
    }


    /**
     * 定时任务：删除超过配置时长的录制视频
     *
     * @throws IOException
     * @throws InterruptedException
     */
    public void deleteFlvExceed() throws IOException, InterruptedException {
        //字典中设置的值
        List<SysDictData> sysCameraRecordTime = dictDataMapper.selectDictDataByType("sys_camera_record_time");
        //开启流
        List<String> rspList = new ArrayList<String>();
        Process proc = Runtime.getRuntime().exec(transcribeFilePath, null, null);
        BufferedReader in = new BufferedReader(new InputStreamReader(proc.getInputStream()));
        PrintWriter out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(proc.getOutputStream())), true);

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        Date date = new Date();
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(date);
        //正数，日期天数加
        //负数，日期天数减                         提前45天
        if (!ObjectUtils.isEmpty(sysCameraRecordTime)) {
            String dictValue = sysCameraRecordTime.get(0).getDictValue();
            calendar.add(Calendar.DAY_OF_YEAR, Integer.parseInt("-" + dictValue));
        } else {
            calendar.add(Calendar.DAY_OF_YEAR, -45);
        }
        date = calendar.getTime();
        //时间格式化为字符串
        String formatD = sdf.format(date);
        log.info("formatD======================================" + formatD);

        //遍历每个进行视频录制的摄像头
        File file = new File(transcribeFilePath);
        if (file.exists()) {
            String s = HttpUtils.sendGet(transcribeFilePath, null);
            if (!StringUtils.isBlank(s) || "null".equals(s)) {
                List<Map<String, Object>> maps = JSON.parseArray(s, Map.class);
                if (ObjectUtils.isEmpty(maps)) {
                    return;
                }
                List<String> commands = new ArrayList<>();
                for (Map<String, Object> map : maps) {
                    Object path = map.get("Path");
                    log.info("path======================================" + path.toString());
                    //查出符合条件的视频文件，删掉
                    if (!ObjectUtils.isEmpty(path)) {
                        String commit = " rm -rf  " + transcribeFilePath + "/" + path.toString() + "/" + formatD + "*";
                        commands.add(commit);
                    }
                }
                // 删除
                for (String line : commands) {
                    out.println(line);
                    log.info("line======================================" + line);
                }
            }
        }
        // 这个命令必须执行，否则in流不结束。
        out.println("exit");
        String rspLine = "";
        while ((rspLine = in.readLine()) != null) {
            System.out.println(rspLine);
            rspList.add(rspLine);
        }
        int i = proc.waitFor();
        log.info("执行结果：{}", i);
        //关闭流
        in.close();
        out.close();
        proc.destroy();
    }


}
